#include "led_strip.h"


uint8_t strips_cfged = 0;
strand_t leds[8];

uint8_t render_frequency = 50;

qthing::led_color_t dummy_handler(uint8_t strip, uint16_t led, qthing::led_color_t last_color) {
    return(last_color);
}

qthing::led_color_handler_t led_handler = dummy_handler;

void render_loop(void *ignored) {
    digitalLeds_initStrands(leds, strips_cfged);
    TickType_t lastWakeTime = xTaskGetTickCount();

    while (true) {
        for (uint8_t strip = 0; strip < strips_cfged; strip++) {
            for (uint16_t led = 0; led < leds[strip].numPixels; led++) {
                pixelColor_t last = leds[strip].pixels[led];
                qthing::led_color_t last_led = {last.r, last.g, last.b, last.w};
                qthing::led_color_t pixel = led_handler(strip, led, last_led);
                leds[strip].pixels[led] = pixelFromRGBW(pixel.r, pixel.g, pixel.b, pixel.w);
            }
        }

        for (uint8_t i = 0; i < strips_cfged; i++) digitalLeds_updatePixels(&leds[i]);
        vTaskDelayUntil(&lastWakeTime, 1000 / render_frequency / portTICK_PERIOD_MS);
    }
}

namespace qthing {
    void add_ledstrip(gpio_num_t gpio, uint16_t num_leds, qthing::led_strip_types_t led_type) {

        int led_type_translated = 0;

        switch (led_type) {
            case qthing::led_strip_types_t::LED_WS2812_V1: led_type_translated = led_types::LED_WS2812_V1; break;
            case qthing::led_strip_types_t::LED_WS2812B_V1: led_type_translated = led_types::LED_WS2812B_V1; break;
            case qthing::led_strip_types_t::LED_WS2812B_V2: led_type_translated = led_types::LED_WS2812B_V2; break;
            case qthing::led_strip_types_t::LED_WS2812B_V3: led_type_translated = led_types::LED_WS2812B_V3; break;
            case qthing::led_strip_types_t::LED_WS2813_V1: led_type_translated = led_types::LED_WS2813_V1; break;
            case qthing::led_strip_types_t::LED_WS2813_V2: led_type_translated = led_types::LED_WS2813_V2; break;
            case qthing::led_strip_types_t::LED_WS2813_V3: led_type_translated = led_types::LED_WS2813_V3; break;
            case qthing::led_strip_types_t::LED_SK6812_V1: led_type_translated = led_types::LED_SK6812_V1; break;
            case qthing::led_strip_types_t::LED_SK6812W_V1: led_type_translated = led_types::LED_SK6812W_V1; break;
        }

        leds[strips_cfged] = {
            .rmtChannel = strips_cfged,
            .gpioNum = gpio,
            .ledType = led_type_translated,
            .brightLimit = 255,
            .numPixels =  num_leds,
            .pixels = nullptr, 
            ._stateVars = nullptr
        };

        gpio_pad_select_gpio(gpio);
        gpio_set_direction(gpio, GPIO_MODE_OUTPUT);
        gpio_set_level(gpio, 0);

        strips_cfged++;
    }

    void set_led_strip_fps(uint8_t fps) {
        render_frequency = fps;
    }

    void enable_leds(uint8_t led_strip_fps) {
        set_led_strip_fps(led_strip_fps);

        // pin this to app cpu as it may consume a lot of cycles
        xTaskCreatePinnedToCore(render_loop, "render_loop", 10000, NULL, 6, NULL, 1);
    }

    void set_color_handler(led_color_handler_t led_color_handler) {
        led_handler = led_color_handler;
    }
}
